package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaCarRecordMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.*;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.iai.v20180301.models.Candidate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-18
 */
@Service
public class TaCarRecordServiceImpl extends ServiceImpl<TaCarRecordMapper, TaCarRecord> implements ITaCarRecordService {
    Logger log = LoggerFactory.getLogger(TaCarRecordServiceImpl.class);

    @Autowired
    private TaCarRecordMapper taCarRecordMapper;

    @Autowired
    private ITaCarService iTaCarService;

    @Autowired
    private ITaPersonService iTaPersonService;

    @Autowired
    private  ITaCustomerMapService iTaCustomerMapService;

    @Autowired
    private ITaFirstUsherRecordService iTaFirstUsherRecordService;

    @Autowired
    private ITaCustomerService iTaCustomerService;

    @Autowired
    private AliFaceUtils aliFaceUtils;

    @Autowired
    private ISysParamService iSysParamService;

    @Autowired
    private ICameraService iCameraService;

    @Autowired
    private ITaVisitorAppointmentService iTaVisitorAppointmentService;

    @Override
    public IPage<TaCarRecord> getPageList(IPage<TaCarRecord> page, String plateNumber, Date recordDate, String customerName, String phone) {
        return taCarRecordMapper.getPageList(page, plateNumber, recordDate, customerName, phone);
    }

    /**
     * 车辆抓拍之后
     * @param taCarRecord
     * @return
     */
    @Override
    public boolean newRecordByDevice(TaCarRecord taCarRecord) {
        boolean workingTime = iSysParamService.isInWorkingTime(LocalDateTime.now());
        if (!workingTime) {
            taCarRecordMapper.insert(taCarRecord);
            iTaCarService.newCarByDevice(taCarRecord);
            return true;
        }

        // 是否内部车辆
        TaCar taCar = iTaCarService.getById(taCarRecord.getPlateNumber());
        if (null != taCar && taCar.getTypeId().equals(CommConstant.TYPE_CAR_INNER)) {
            taCarRecordMapper.insert(taCarRecord);
            iTaCarService.newCarByDevice(taCarRecord);
            return true;
        }

        // 到客户中查询是否有相关人员
        TaCustomer taCustomer = null;
        List<TaCustomer> taCustomerList = iTaCustomerService.getSomeCustMayBe(taCarRecord.getPlateNumber());

        // 如果没有对应人员
        Integer personId = null;
        if (null == taCustomerList || taCustomerList.size() == 0) {
            // 再去人脸库比对一次
            if (null != taCarRecord.getDriverImage() && aliFaceUtils.detectFace(taCarRecord.getDriverImage())) {
                try {
                    Float threshold = iSysParamService.getSamePersonThreshold();
                    JSONObject result = aliFaceUtils.searchFace(taCarRecord.getDriverImage());

                    if (null != result && result.getFloat("rate") * 100 >= threshold) {
                        personId = Integer.valueOf(result.getString("personId"));
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            // 依据人脸与预约判断
            // 人脸优先级高
            if (null != personId && personId > 0) {
                // 是否内部人员
                TaPerson taPerson = iTaPersonService.getByIdOrReal(personId);
                if (null != taPerson && null != taPerson.getTypeId() && taPerson.getTypeId() == CommConstant.TYPE_INNER_PERSON) {
                    taCarRecordMapper.insert(taCarRecord);
                    iTaCarService.newCarByDevice(taCarRecord);
                    return true;
                }

                // 找到对应人脸
                taCustomer = iTaCustomerService.getSameByPerson(personId);
                if (null != taCustomer) {
                    // 尝试更新部分客户信息
                    iTaCustomerService.updataByCarRecord(taCustomer, taCarRecord);
                }
            }

            // 去预约记录查询一次
            List<TaVisitorAppointment> appointments = iTaVisitorAppointmentService.getAppointmentsByPlateNumber(taCarRecord.getPlateNumber());

            if (null == taCustomer) {
                if (null != appointments && appointments.size() > 0) {
                    taCustomer = iTaCustomerService.newCustByCarDevice(taCarRecord, appointments.get(0));
                    taCarRecord.setPhone(appointments.get(0).getPhone());
                    taCarRecord.setCustomerName(appointments.get(0).getPersonName());
                    taCarRecord.setPersonNum(appointments.get(0).getPersonNum());
                } else {
                    // 只有有预约才新建客户
//                    taCustomer = iTaCustomerService.newCustByCarDevice(taCarRecord);
                }
            }
        } else {
            // 对应客户, 先随机取一条
            taCustomer = taCustomerList.get(0);
        }

        // 新增人脸
        if (null != taCarRecord.getDriverImage() && aliFaceUtils.detectFace(taCarRecord.getDriverImage())) {
            TaPerson person = new TaPerson();
            if (null != taCustomer) {
                person.setRealId(taCustomer.getCustomerId());
                person.setPhone(taCustomer.getPhone());
                person.setName(taCustomer.getCustomerName());
                if (null != taCustomer.getSex()) {
                    person.setSex(String.valueOf(taCustomer.getSex()));
                }
            }
            person.setTypeId(CommConstant.TYPE_NEW_PERSON);
            person.setStatus(StatusUtils.Ready);
            person.setCreateDate(LocalDateTime.now());
            person.setLastVisiteDate(LocalDateTime.now());
//            person.setLastVisiteDevice();
            person.setAvatar(taCarRecord.getDriverImage());
            if (null != personId) {
                TaPerson psn = iTaPersonService.getById(personId);
                if (null != psn) {
                    person.setRealId(psn.getRealId());
                }
            }

            boolean success = iTaPersonService.save(person);

            if (success && null != taCustomer) {
                TaCustomerMap mp = new TaCustomerMap();
                mp.setCustomerId(taCustomer.getCustomerId());
                mp.setPersonId(person.getPersonId());
                iTaCustomerMapService.save(mp);
            }

            if (null == person.getRealId()) {
                person.setRealId(person.getPersonId());
                iTaPersonService.updateById(person);
            }

            try {
                iCameraService.dispatchPerson(person);
            } catch (Exception e) {
                log.error("下发驾驶员人脸失败: {}", e.getMessage());
                e.printStackTrace();
            }
        }

        // 先入库
        taCarRecordMapper.insert(taCarRecord);
        iTaCarService.newCarByDevice(taCarRecord);

        // 生成保安迎宾记录, 并通知小程序
        TaFirstUsherRecord taFirstUsherRecord = iTaFirstUsherRecordService.newRecordByDevice(taCarRecord);
        iTaFirstUsherRecordService.notifyMiniapp(taFirstUsherRecord);

        // 如果客户有固定置业, 推送消息给置业
        if (null != taCustomer && null != taCustomer.getConsultantId()) {
            try {
                iTaCustomerService.notifyConsultant(taCustomer);
            } catch (Exception e) {
                log.error("发送消息给置业失败: {}", e.getMessage());
            }
        }

        return true;
    }

    @Override
    public boolean updateRecordByMiniapp(TaCarRecord taCarRecord) {
        // 先暂时只做更新动作
        return taCarRecordMapper.updateById(taCarRecord) > 0;
    }

    @Override
    public List<TaCarRecord> getTimeline() {
        LocalDateTime now = LocalDateTime.now();
        String dayBegin = DateUtils.localDatetime2day(now) + " 00:00:00";

//        QueryWrapper<TaCarRecord> query = new QueryWrapper<>();
//        query.ge("create_date", dayBegin);
//        query.groupBy("plate_number");
//        query.orderByDesc("create_date");
//        List<TaCarRecord> result = taCarRecordMapper.selectList(query);

        List<TaCarRecord> result = taCarRecordMapper.getCarRecordTimeLine(dayBegin);

        return result;
    }
}
