package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.*;
import com.huiju.welcome.model.*;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaMainUsherRecord;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaMainUsherRecordService;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@Slf4j
@Service
public class TaMainUsherRecordServiceImpl extends ServiceImpl<TaMainUsherRecordMapper, TaMainUsherRecord> implements ITaMainUsherRecordService {
    @Autowired
    private TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    private TaCustomerMapper taCustomerMapper;

    @Autowired
    TaVisitorAppointmentMapper taVisitorAppointmentMapper;

    @Autowired
    TaFollowupRecordMapper taFollowupRecordMapper;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    AliFaceUtils aliFaceUtils;

    @Autowired
    MiniApp miniApp;

    @Override
    public boolean updateNormal(List<TaMainUsherRecord> records) {
        if (records == null || records.size() == 0) return true;

        List<Integer> recs = new ArrayList<>();
        for (TaMainUsherRecord rec: records) {
            recs.add(rec.getRecId());
        }

        UpdateWrapper<TaMainUsherRecord> wrapper = new UpdateWrapper<>();
        wrapper.set("status", StatusUtils.Normal);
        wrapper.in("rec_id", recs);

        int row = taMainUsherRecordMapper.update(new TaMainUsherRecord(), wrapper);
        return row >= 0;
    }

    @Override
    public boolean newRecordByDeviceDeprecated(TaCustomer taCustomer, TaPerson taPerson) {
        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer, null);
        taMainUsherRecord.setStatus(StatusUtils.Ready);
        taMainUsherRecord.setPersonId(taPerson.getRealId());
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        try {
            Action act = Action.create(Action.TYPE_NEW_MAIN_USHER, JSONObject.toJSONString(taMainUsherRecord, SerializerFeature.WriteMapNullValue));
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {e.printStackTrace();}

        // 通知对应的置业
        if (null != taCustomer.getConsultantId()) {
            try {
                notifyConsultant(taMainUsherRecord, taCustomer);
            } catch (Exception e) {e.printStackTrace();}
        }

        return row > 0;
    }

    @Override
    public boolean newRecordByConsultant(TaCustomer taCustomer, JSONArray follows) {
        if (StringUtils.isEmpty(taCustomer.getRemark()) && null == follows) {
            return false;
        }

        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer, null);
        taMainUsherRecord.setCustomerId(taCustomer.getCustomerId());
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        insertFollows(follows, taMainUsherRecord);

        return row > 0;
    }

    private boolean insertFollows(JSONArray follows, TaMainUsherRecord taMainUsherRecord) {
        if (null != follows && follows.size() > 0) {
            for (int i = 0; i < follows.size(); i ++) {
                TaFollowupRecord rec = new TaFollowupRecord();
                rec.setRecId(taMainUsherRecord.getRecId());
                rec.setPersonId(follows.getJSONObject(i).getInteger("personId"));
                rec.setAvatar(follows.getJSONObject(i).getString("avatar"));
                taFollowupRecordMapper.insert(rec);
            }
        }

        return true;
    }

    private TaMainUsherRecord newRecordByCust(TaCustomer taCustomer, SysUser user) {
        TaMainUsherRecord taMainUsherRecord = new TaMainUsherRecord();
        taMainUsherRecord.setCustomerId(taCustomer.getCustomerId());
        taMainUsherRecord.setPersonId(taCustomer.getCustomerId());
        taMainUsherRecord.setPersonName(taCustomer.getCustomerName());
        taMainUsherRecord.setAvatar(taCustomer.getAvatar());
        taMainUsherRecord.setPhone(taCustomer.getPhone());
        taMainUsherRecord.setAppointment(taCustomer.getApointment());
        taMainUsherRecord.setChannel(taCustomer.getChannel());
        taMainUsherRecord.setPurpose(taCustomer.getPurpose());
        taMainUsherRecord.setPlateNumber(taCustomer.getPlateNumber());
        taMainUsherRecord.setCarModel(taCustomer.getCarModel());
        taMainUsherRecord.setColor(taCustomer.getCarColor());
        taMainUsherRecord.setVisiteDate(LocalDateTime.now());
        taMainUsherRecord.setRemark(taCustomer.getRemark());

        if (null != user) {
            taMainUsherRecord.setConsultantId(user.getUserId());
            taMainUsherRecord.setRealtyConsultant(user.getUsername());
        } else {
            taMainUsherRecord.setConsultantId(taCustomer.getConsultantId());
            taMainUsherRecord.setRealtyConsultant(taCustomer.getRealtyConsultant());
        }

        if (StringUtils.isEmpty(taCustomer.getPhone())) {
            taMainUsherRecord.setStatus(StatusUtils.Ready);
        } else {
            taMainUsherRecord.setStatus(StatusUtils.Normal);
        }

        return taMainUsherRecord;
    }

    @Override
    public IPage<TaMainUsherRecord> taMainUsherRecordList(IPage<TaMainUsherRecord> pg,String phone,String platNumber,String personName,Integer appointment) {
        IPage<TaMainUsherRecord>  taMainUsherRecordList= taMainUsherRecordMapper.taMainUsherRecordList(pg,phone,platNumber,personName,appointment);
        return taMainUsherRecordList;
    }

    @Override
    public void notifyConsultant(TaMainUsherRecord taMainUsherRecord, TaCustomer taCustomer) throws Exception {
        Integer realConsultant = taCustomer.getConsultantId();
        Integer proxyConsultant = taMainUsherRecord.getConsultantId();

        // 查找置业微信ID
        String openid = taWechatUserMapper.getOpenidOf(proxyConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
        if (StringUtils.isEmpty(openid)) {
            throw new Exception("置业尚未绑定微信小程序");
        }

        String visitorName = taMainUsherRecord.getPersonName();
        String phone = taMainUsherRecord.getPhone();
        Integer customerId = taMainUsherRecord.getCustomerId();

        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + String.valueOf(customerId);
        String tplId = miniApp.getTpls().getVisitor();

        // 如果是客户对应的置业进行接待
        if (realConsultant.equals(proxyConsultant)) {
            List<Object> data = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 请及时接待!");
            }};

            miniApp.sendTPLMessage(tplId, openid, page, data);
        } else {
            // 否则, 给原置业发消息, 给接待置业发消息

            // 原绑定置业消息
            String rcOpenid = taWechatUserMapper.getOpenidOf(realConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(openid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }

            SysUser usr1 = sysUserMapper.selectById(proxyConsultant);
            List<Object> dt1 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 已被安排给 " + usr1.getUsername() + " 接待!");
            }};
            miniApp.sendTPLMessage(tplId, rcOpenid, page, dt1);



            // 代理置业
            SysUser usr2 = sysUserMapper.selectById(realConsultant);
            List<Object> dt2 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add(usr2.getUsername() + " 有新客户来访, 请协助接待一下!");
            }};
            miniApp.sendTPLMessage(tplId, openid, page, dt2);
        }
    }

    @Override
    public ResponseBean customerPage(int pageNum, int pageSize, String customerName, String phone, String platNumber, Integer agentUserId) {
        ResponseBean response = new ResponseBean();
        IPage<TaCustomer> pg = new Page<>(pageNum, pageSize);
        IPage<TaCustomer> result = taCustomerMapper.customerPage(pg,customerName,phone,platNumber,agentUserId);
        response.addSuccess(result);
        return response;
    }

    @Override
    public boolean newRecordByMiniapp(TaMainUsherRecord taMainUsherRecord) throws Exception {
        Integer consultantId = taMainUsherRecord.getConsultantId();
        if (null == consultantId || consultantId < 1) {
            throw new Exception("置业顾问不能为空");
        }

        if (StringUtils.isEmpty(taMainUsherRecord.getPhone())) {
            throw new Exception("客户电话不能为空");
        }

        taMainUsherRecord.setStatus(StatusUtils.Normal);

        // 新增客户
        iTaCustomerService.newCustByMainUsher(taMainUsherRecord, true);

        return taMainUsherRecordMapper.insert(taMainUsherRecord) > 0;
    }

    @Override
    public boolean updateByMiniapp(TaMainUsherRecord taMainUsherRecord) throws Exception {
        Integer consultantId = taMainUsherRecord.getConsultantId();
        if (null == consultantId || consultantId < 1) {
            throw new Exception("置业顾问不能为空");
        }

        if (StringUtils.isEmpty(taMainUsherRecord.getPhone())) {
            throw new Exception("客户电话不能为空");
        }

        taMainUsherRecord.setStatus(StatusUtils.Normal);

        // 是否更新了置业
        TaCustomer taCustomer = null;
        if (null != taMainUsherRecord.getCustomerId()) {
            taCustomer = iTaCustomerService.getById(taMainUsherRecord.getCustomerId());
        } else if (null != taMainUsherRecord.getPhone() || null != taMainUsherRecord.getPlateNumber()) {
            // 尝试使用手机号去查找
            taCustomer = iTaCustomerService.getByPhoneOrPlate(taMainUsherRecord.getPhone(), taMainUsherRecord.getPlateNumber());
        } else if (null != taMainUsherRecord.getAvatar()) {
            // 依据人脸查询
            JSONObject result = aliFaceUtils.searchFace(taMainUsherRecord.getAvatar());
            if (null != result) {
                Integer personId = Integer.valueOf(result.getString("personId"));
                taCustomer = iTaCustomerService.getSameByPerson(personId);
            }
        }

        if (null == taCustomer && null == taMainUsherRecord.getCustomerId()) {
            taCustomer = iTaCustomerService.newCustByMainUsher(taMainUsherRecord, false);
        }

        Integer originConsultant = taCustomer.getConsultantId();

        // 是否需要更新客户信息
        // 只更新空内容
        if (StringUtils.isEmpty(taCustomer.getCustomerName()) || StringUtils.isEmpty(taCustomer.getPhone())) {
            if (StringUtils.isEmpty(taCustomer.getCustomerName())) {
                taCustomer.setCustomerName(taMainUsherRecord.getPersonName());
            }

            if (StringUtils.isEmpty(taCustomer.getPhone())) {
                taCustomer.setPhone(taMainUsherRecord.getPhone());
            }

            if (null == taCustomer.getSex()) {
                taCustomer.setSex(taMainUsherRecord.getSex());
            }

            if (StringUtils.isEmpty(taCustomer.getAvatar())) {
                taCustomer.setAvatar(taMainUsherRecord.getAvatar());
            }

            if (StringUtils.isEmpty(taCustomer.getPlateNumber())) {
                taCustomer.setPlateNumber(taMainUsherRecord.getPlateNumber());
            }

            if (StringUtils.isEmpty(taCustomer.getCarModel())) {
                taCustomer.setCarModel(taMainUsherRecord.getCarModel());
            }

            iTaCustomerService.updateById(taCustomer);
        }

        int row = taMainUsherRecordMapper.updateById(taMainUsherRecord);

        // 更新人员信息
        iTaPersonService.updateByMainUsher(taMainUsherRecord);

        // 如果原来置业为空
        if (null == originConsultant || originConsultant < 1) {
            iTaCustomerService.updateConsultant(taCustomer, consultantId);
        } else if (!consultantId.equals(originConsultant)) {
            // 如果临时替换跟踪置业
            // 那么需要给原始置业, 以及临时置业各自发一条消息
            String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + String.valueOf(taCustomer.getCustomerId());
            String tplId = miniApp.getTpls().getVisitor();

            // 原绑定置业
            String rcOpenid = taWechatUserMapper.getOpenidOf(originConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(rcOpenid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }
            SysUser rcUser = sysUserMapper.selectById(originConsultant);

            // 临时置业
            String proxyOpenid = taWechatUserMapper.getOpenidOf(consultantId, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(proxyOpenid)) {
                throw new Exception("置业尚未绑定微信小程序");
            }
            SysUser proxyUser = sysUserMapper.selectById(consultantId);

            // 通知原置业, 客户被临时置业接待
            List<Object> dt1 = new ArrayList<Object>();
            dt1.add(taCustomer.getCustomerName());
            dt1.add(taCustomer.getPhone());
            dt1.add("您有新客户来访, 已被安排给 " + proxyUser.getUsername() + " 接待!");

            miniApp.sendTPLMessage(tplId, rcOpenid, page, dt1);

            // 通知临时置业
            List<Object> dt2 = new ArrayList<Object>();
            dt2.add(taCustomer.getCustomerName());
            dt2.add(taCustomer.getPhone());
            dt2.add(rcUser.getUsername() + " 有新客户来访, 请协助接待一下!");

            miniApp.sendTPLMessage(tplId, proxyOpenid, page, dt2);
        }

        return row > 0;
    }

    @Override
    public boolean deleteRecord(Integer id) {
        UpdateWrapper<TaMainUsherRecord> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("rec_id", id);
        int row = taMainUsherRecordMapper.update(new TaMainUsherRecord(), updateWrapper);

        return row > 0;
    }

    @Override
    public boolean newRecordByDevice(TaPerson taPerson, TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = null;
        if (null != taCustomer) {
            taMainUsherRecord = newRecordByCust(taCustomer, null);
        } else {
            taMainUsherRecord = newRecordByPerson(taPerson);
        }

        taMainUsherRecord.setStatus(StatusUtils.Ready);
        taMainUsherRecord.setPersonId(taPerson.getRealId());
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        try {
            Action act = Action.create(Action.TYPE_NEW_MAIN_USHER, JSONObject.toJSONString(taMainUsherRecord, SerializerFeature.WriteMapNullValue));
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {e.printStackTrace();}

        return row > 0;
    }

    @Override
    public boolean updateByCustomer(TaCustomer taCustomer, JSONArray follows, String userId) {
        TaMainUsherRecord taMainUsherRecord = getTodayRecord(taCustomer.getCustomerId());

        if (null == taMainUsherRecord) {
            Integer personId = taWechatUserMapper.selectById(userId).getPersonId();
            taMainUsherRecord = newRecordByCust(taCustomer, sysUserMapper.selectById(personId));

            int row = taMainUsherRecordMapper.insert(taMainUsherRecord);
            if (row < 1) {
                log.error("新增跟进记录失败");
                return false;
            }
        } else {
            String remark = taMainUsherRecord.getRemark();

            if (null == remark) {
                remark = taCustomer.getRemark();
            } else if (!StringUtils.isEmpty(taCustomer.getRemark())) {
                remark += "\n\n---" + DateUtils.cutSecond(LocalDateTime.now())+ "---" + "\n" + taCustomer.getRemark();
            }
            taMainUsherRecord.setRemark(remark);

            int row = taMainUsherRecordMapper.updateById(taMainUsherRecord);
            if (row < 1) {
                log.error("更新跟进记录失败");
                return false;
            }
        }

        insertFollows(follows, taMainUsherRecord);

        return true;
    }

    @Override
    public TaMainUsherRecord getTodayRecord(Integer customerId) {
        LocalDateTime now = LocalDateTime.now();
        String today = DateUtils.localDatetime2day(now);

        QueryWrapper<TaMainUsherRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("visite_date", today);
        queryWrapper.eq("customer_id", customerId);
        queryWrapper.orderByDesc("visite_date");
        queryWrapper.last("limit 1");

        return taMainUsherRecordMapper.selectOne(queryWrapper);
    }
    
    @Override
    public boolean updateMainUsherRecord(TaMainUsherRecord taMainUsherRecord, Integer customerId, String userId) {
	    List<TaFollowupRecord> taFollowupRecordList = taMainUsherRecord.getFollows();
	    TaCustomer taCustomer = taCustomerMapper.selectById(customerId);
        if (null == taMainUsherRecord.getRecId()) {
            Integer personId = taWechatUserMapper.selectById(userId).getPersonId();
            taMainUsherRecord = newRecordByCust(taCustomer, sysUserMapper.selectById(personId));
        
            int row = taMainUsherRecordMapper.insert(taMainUsherRecord);
            if (row < 1) {
                log.error("新增跟进记录失败");
                return false;
            }
        } else {
            TaMainUsherRecord origin = taMainUsherRecordMapper.selectById(taMainUsherRecord.getRecId());
            String remark = taMainUsherRecord.getRemark();
        
            if (null == remark) {
                remark = origin.getRemark();
            }
            else if (!StringUtils.isEmpty(origin.getRemark()) && !remark.equals(origin.getRemark())) {
                remark += "\n\n---" + DateUtils.cutSecond(LocalDateTime.now())+ "---" + "\n" + origin.getRemark();
            }
            taMainUsherRecord.setRemark(remark);
        
            int row = taMainUsherRecordMapper.updateById(taMainUsherRecord);
            if (row < 1) {
                log.error("更新跟进记录失败");
                return false;
            }
            //删除随访人员头像
	        QueryWrapper<TaFollowupRecord> queryWrapper = new QueryWrapper<>();
	        queryWrapper.eq("rec_id",taMainUsherRecord.getRecId());
	        taFollowupRecordMapper.delete(queryWrapper);
        }

        if (null != taFollowupRecordList) {
            for (TaFollowupRecord follow : taFollowupRecordList) {
                follow.setRecId(taMainUsherRecord.getRecId());
                taFollowupRecordMapper.insert(follow);
            }
        }
    
        return true;
    }

    @Override
    public TaMainUsherRecord newRecordByFirstUsher(TaFirstUsherRecord taFirstUsherRecord, TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = new TaMainUsherRecord();
        taMainUsherRecord.setColor(taFirstUsherRecord.getColor());
        taMainUsherRecord.setPlateNumber(taFirstUsherRecord.getPlateNumber());
        taMainUsherRecord.setStatus(StatusUtils.Normal);
        taMainUsherRecord.setAvatar(taFirstUsherRecord.getDriverImage());
        taMainUsherRecord.setPhone(taFirstUsherRecord.getPhone());
        taMainUsherRecord.setVisiteDate(taFirstUsherRecord.getVisiteDate());
        taMainUsherRecord.setCarModel(taFirstUsherRecord.getCarModel());
        taMainUsherRecord.setChannel(taFirstUsherRecord.getChannel());
        taMainUsherRecord.setPurpose(taFirstUsherRecord.getPurpose());
        taMainUsherRecord.setCarImage(taFirstUsherRecord.getCarImage());

        if (null != taCustomer) {
            taMainUsherRecord.setPersonId(taCustomer.getCustomerId());
            taMainUsherRecord.setCustomerId(taCustomer.getCustomerId());
            taMainUsherRecord.setConsultantId(taCustomer.getConsultantId());
            taMainUsherRecord.setPersonName(taCustomer.getCustomerName());
            taMainUsherRecord.setRealtyConsultant(taCustomer.getRealtyConsultant());
            taMainUsherRecord.setSex(taCustomer.getSex());
            taMainUsherRecord.setPhone(taCustomer.getPhone());
        }

        taMainUsherRecordMapper.insert(taMainUsherRecord);
        return taMainUsherRecord;
    }

    private TaMainUsherRecord newRecordByPerson(TaPerson taPerson) {
        TaMainUsherRecord taMainUsherRecord = new TaMainUsherRecord();
        taMainUsherRecord.setPersonName(taPerson.getName());
        taMainUsherRecord.setAvatar(taPerson.getAvatar());
        taMainUsherRecord.setPhone(taPerson.getPhone());
        taMainUsherRecord.setVisiteDate(LocalDateTime.now());

        if (StringUtils.isEmpty(taPerson.getPhone())) {
            taMainUsherRecord.setStatus(StatusUtils.Ready);
        } else {
            taMainUsherRecord.setStatus(StatusUtils.Normal);
        }

        return taMainUsherRecord;
    }
}
