package com.huiju.welcome.utils;

import com.alibaba.fastjson.JSONObject;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.service.ITaMessageRecordService;
import com.huiju.welcome.service.ITaMiniFormidsService;
import lombok.Data;
import org.apache.http.HttpStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Data
@Component
@ConfigurationProperties(prefix = "miniapp")
public class MiniApp {
    private Logger log = LoggerFactory.getLogger(MiniApp.class);

    @Autowired
    ITaMiniFormidsService iTaMiniFormidsService;

    @Autowired
    ITaMessageRecordService iTaMessageRecordService;

    private String url;

    private String appid;

    private String secret;

    private Tpl tpls;


    public ResponseBean getOpenid(String code) {
        String api = url + "/api/miniapp/" + appid + "/login?code=" + code;
        RestTemplate restTemplate = new RestTemplate();
        String resp = restTemplate.exchange(api,HttpMethod.resolve("GET"), new HttpEntity<String>(null, null),String.class).getBody();

        JSONObject result = JSONObject.parseObject(resp);
        Integer respCode = result.getInteger("code");
        if (null == respCode || respCode.intValue() != 0) {
            log.error("请求小程序服务接口失败, 请求地址 " + api);
            log.error("请求结果: " + resp);
            return ResponseBean.error("调用小程序接口失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        return ResponseBean.success(result.getJSONObject("data"));
    }

    public ResponseBean sendTPLMessage(String tplId, String toUser, String page, List<Object> data) {
        boolean needSend = iTaMessageRecordService.saveMessageOrNot(toUser, tplId, page, data, false);
        if (!needSend) return ResponseBean.success("");

        String formId = iTaMiniFormidsService.getFormidBy(toUser, true);
        if (null == formId) {
            return ResponseBean.error("发送模板消息失败, 没有可用的 formid ", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        String api = String.format("/api/%s/tpl/%s/form/%s/to/%s", appid, tplId, formId, toUser);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        Map<String, Object> sendData = new HashMap<String, Object>();
        sendData.put("page", page);
        sendData.put("data", data);

        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<Map<String, Object>> request = new HttpEntity<Map<String, Object>>(sendData, headers);
        String resp ="";
        try{
            resp = restTemplate.postForEntity(url+api, request, String.class).getBody();
        }catch (Exception e){
            log.error(e.getMessage());
        }

        JSONObject result = JSONObject.parseObject(resp);
        Integer respCode = result.getInteger("code");
        if (null == respCode || respCode.intValue() != 0) {
            log.error("请求小程序服务接口失败, 请求地址 " + api);
            log.error("请求结果: " + resp);
            return ResponseBean.error("调用小程序接口失败", HttpStatus.SC_INTERNAL_SERVER_ERROR);
        }

        iTaMessageRecordService.saveMessageOrNot(toUser, tplId, page, data, true);

        return ResponseBean.success("");
    }

    @Data
    public static class Tpl {
        private String visitor;
    }
}
